/* copy boot sector */

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <bios.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <io.h>
#include <ctype.h>
#include <dos.h>

static struct diskinfo_t info;
static struct boot_struct {
        unsigned char   jmp[3];         // jump instruction
        char    oem[8];                 // OEM name;
        unsigned int    bytes_per_sector;
        unsigned char   sector_per_cluster;
        unsigned int    reserved_sectors;
        unsigned char   number_of_fats;
        unsigned int    number_of_dir_entries;
        unsigned int    sectors_in_logical_image;
        unsigned char   media_id;
        unsigned int    sectors_in_1_fat;
        unsigned int    sectors_per_track;
        unsigned int    number_of_heads;
        unsigned int    number_of_hidden_sectors;
        } *bsp;
static char *sector_from_A;

static unsigned drv = 0;    // drive 0=A 1=B

static char myname[32];
static char *batparms[4] = {NULL,NULL,NULL,NULL};
static int batparmX = 0;

int dup_onto_C(char *fn)
    {
    char fname[16];
    char fnameA[20];
    char cmd[64];
    char *p;
    unsigned old_attr;

    memcpy(fname,fn,8);
    p=&fname[7];
    while (*p == ' ') {
        p--;
        if (p < fname) {
            printf("Disk Empty\n");
            return 1;
            }
        }
    p++;
    *p++ = '.';
    memcpy(p,fn+8,3);
    p+=2;
    while (*p == ' ')
        p--;
    p++;
    *p = '\0';
    sprintf(fnameA,"%c:\\%s",(drv+'A'),fname);
    if (_dos_getfileattr(fnameA,&old_attr)) {
        printf("Could not get attr of %s\n",fnameA);
        return 1;
        }
    if (_dos_setfileattr(fnameA,_A_RDONLY)) {
        printf("Could not set attr of %s\n",fnameA);
        return 1;
        }
    strcpy(cmd,"copy ");
    strcat(cmd,fnameA);
    printf("%s\n",cmd);
    system(cmd);
    batparms[batparmX++]=strdup(fname);
    if (_dos_setfileattr(fnameA,old_attr)) {
        printf("Could not reset attr of %s\n",fnameA);
        return 1;
        }
    return 0;
    }

int get_ibmbio(void)
    {
    unsigned int rc;
    int target, headss;

    bsp = (struct boot_struct *)sector_from_A;
    target = 1 + (bsp->sectors_in_1_fat * bsp->number_of_fats);
    headss = target / bsp->sectors_per_track;
    info.nsectors = 1;      // one sector at a time
    info.buffer = sector_from_A;   // the data buffer
    info.drive  = drv;       // drive
    info.head   = headss % bsp->number_of_heads;
    info.track  = headss / bsp->number_of_heads;
    info.sector = 1 + (target % bsp->sectors_per_track);
    rc = _bios_disk(_DISK_READ,&info);
    if (rc > 255) {
        printf("Read Error %X\n",rc);
        return 2;
        }
    if (*sector_from_A) {
        dup_onto_C(sector_from_A);
        dup_onto_C(sector_from_A+32);
        }
    else {
        printf("Disk %c is Empty\n",'A'+drv);
        return 1;
        }
    return 0;
    }

int do_get(void)
    {
    int i, fh;
    char *p;
    char fname[32];
    char batname[32];
    FILE *fp;

    p=fname;
    for (i=0;i<8;i++) {
        if (isalnum(bsp->oem[i])) {
            *p = bsp->oem[i];
            p++;
            }
        if (bsp->oem[i]=='.') {
            *p = '_';
            p++;
            }
        }
    *p = '\0';
    strcpy(batname,fname);
    strcat(batname,".bat");
    strcat(fname,".bot");
    fh = open(fname,O_WRONLY|O_BINARY|O_CREAT,S_IWRITE|S_IREAD);
    if (fh == -1) {
        perror(fname);
        return 4;
        }
    i = write(fh,sector_from_A,bsp->bytes_per_sector);
    if (i == -1) {
        perror(fname);
        return 6;
        }
    close(fh);
    printf("\nBoot sector written to %s\n",fname);
    batparms[batparmX++]=strdup(fname);
    get_ibmbio();
    if ((fp=fopen(batname,"w")) != NULL) {
        fprintf(fp,"REM bat file to install %s\n",fname);
        fprintf(fp,"REM\n");
        fprintf(fp,"REM parameter 1 == target drive letter\n");
        fprintf(fp,"REM   usually A or B\n");
        fprintf(fp,"REM\n");
        fprintf(fp,"REM parameter 2 (optional) == directory name of copyboot.exe\n");
        fprintf(fp,"REM                            (with trailing \\)\n");
        fprintf(fp,"REM                            only used if copyboot is not\n");
        fprintf(fp,"REM                            in the PATH\n");
        fprintf(fp,"REM\n");
        fprintf(fp,"REM -------------------------------------\n");
        fprintf(fp,"%%2%s put %%1",myname);
        for (i=0;i<batparmX;i++) {
            fprintf(fp," %s",batparms[i]);
            }
        fprintf(fp,"\n");
        fprintf(fp,"REM -------------------------------------\n");
        printf("\nInstall .BAT written to %s\n",batname);
        }
    return 0;
    }


int do_put(int argc, char *fname, char *sysfile1, char *sysfile2)
    {
    int i, fh;
    unsigned int rc;
    char *sector_from_C;
    char cmd[64];

    fh = open(fname,O_RDONLY|O_BINARY,0);
    if (fh == -1) {
        perror(fname);
        return 4;
        }
    sector_from_C = malloc(bsp->bytes_per_sector);
    if (sector_from_C == NULL)
        return 8;
    i = read(fh,sector_from_C,bsp->bytes_per_sector);
    if (i == -1) {
        perror(fname);
        return 6;
        }
    close(fh);
    i = sizeof(struct boot_struct) - 11;
    memcpy(sector_from_C+11,&(bsp->bytes_per_sector),i); // copy A:'s BPB to stored boot
    _bios_disk(_DISK_RESET,&info);
    bsp = (struct boot_struct *)sector_from_A;
    info.drive = drv;       // drive A
    info.nsectors = 1;      // one sector at a time
    info.buffer = sector_from_C;   // the data buffer
    info.head = 0;
    info.track = 0;
    info.sector = 1;
    rc = _bios_disk(_DISK_WRITE,&info);
    if (rc > 255) {
        printf("write Error %X\n",rc);
        return 2;
        }
    printf("Boot record on drive %c: updated\n",(drv+'A'));
    if (argc > 1) {
        sprintf(cmd,"copy %s %c:*.*  ",sysfile1,(drv+'A'));
        printf("%s\n",cmd);
        system(cmd);
        sprintf(cmd,"%c:\\%s",(drv+'A'),sysfile1);
        if (_dos_setfileattr(cmd,_A_SYSTEM|_A_HIDDEN|_A_RDONLY)) {
            printf("Could not set attr of %s\n",cmd);
            return 1;
            }
        }
    if (argc > 2) {
        sprintf(cmd,"copy %s %c:*.*  ",sysfile2,(drv+'A'));
        printf("%s\n",cmd);
        system(cmd);
        sprintf(cmd,"%c:\\%s",(drv+'A'),sysfile2);
        if (_dos_setfileattr(cmd,_A_SYSTEM|_A_HIDDEN|_A_RDONLY)) {
            printf("Could not set attr of %s\n",cmd);
            return 1;
            }
        }
    return 0;
    }

int do_dump(char *p1, char *p2, char *p3)
    {
    unsigned int rc;

    bsp = (struct boot_struct *)sector_from_A;
    info.drive = 0;         // drive A
    info.nsectors = 1;      // one sector at a time
    info.buffer = sector_from_A;   // the data buffer
    info.head = atoi(p1);
    info.track = atoi(p2);
    info.sector = atoi(p3);
    rc = _bios_disk(_DISK_READ,&info);
    if (rc > 255) {
        printf("Read Error %X\n",rc);
        return 2;
        }
    printf("file1: [%s]\n",sector_from_A);
    printf("file2: [%s]\n",sector_from_A+32);
    return 0;
    }


int main(int argc, char *argv[])
    {
    unsigned int rc;
    int i, retrys;
    char dum[64];

    if (argc < 3)
        {
        printf("Usage:\n");
        // ------   0  ---- 1 --- 2 -------- 3 ----------
        printf("   copyboot get <Drive>\n");
        printf("      or\n");
        printf("   copyboot put <Drive> <boot-file-name> [<system-file-1> [<system-file-2>]]\n\n");
        return 1;
        }
    if (stricmp(argv[0],"C"))
        _splitpath(argv[0],dum,dum,myname,dum);
    else
        strcpy(myname,"CopyBoot");
    *argv[2] = toupper(*argv[2]);
    if ((*argv[2] < 'A') || (*argv[2] > 'D') || (strlen(argv[2])>1) ) {
        printf ("Invalid drive leter %s\n",argv[2]);
        return 1;
        }
    drv = *argv[2] - 'A';
    sector_from_A = malloc(4096);
    if (sector_from_A == NULL)
        return 99;
    info.drive = drv;       // drive A
    _bios_disk(_DISK_RESET,&info);
    _bios_disk(_DISK_STATUS,&info);
    if (stricmp("dump",argv[1])==0) {
        i = do_dump(argv[2],argv[3],argv[4]);
        return 0;
        }
    bsp = (struct boot_struct *)sector_from_A;
    info.drive = drv;       // drive A
    info.nsectors = 1;      // one sector at a time
    info.buffer = sector_from_A;   // the data buffer
    info.head = 0;
    info.track = 0;
    info.sector = 1;
    for (retrys=0;retrys<3;retrys++)
        {
        rc = _bios_disk(_DISK_READ,&info);
        rc = rc >> 8;
        if (rc==0)
            break;
        }
    if (rc) {
        printf("Read Error %X\n",rc);
        return 2;
        }
    printf("\n\nCurrent drive %c:\n\n",(drv+'A'));
    printf("Jump %X %X %X\n",bsp->jmp[0],bsp->jmp[1],bsp->jmp[2]);
    printf("OEM name:                 %s\n",bsp->oem);
    printf("sector size:              %d\n",bsp->bytes_per_sector);
    printf("cluster size:             %d\n",bsp->sector_per_cluster);
    printf("reserved sectors:         %d\n",bsp->reserved_sectors);
    printf("number_of_fats:           %d\n",bsp->number_of_fats);
    printf("number_of_dir_entries:    %d\n",bsp->number_of_dir_entries);
    printf("sectors_in_logical_image: %d\n",bsp->sectors_in_logical_image);
    printf("media_id:                 %X\n",bsp->media_id);
    printf("sectors_in_1_fat:         %d\n",bsp->sectors_in_1_fat);
    printf("sectors_per_track:        %d\n",bsp->sectors_per_track);
    printf("number_of_heads:          %d\n",bsp->number_of_heads);
    printf("number_of_hidden_sectors: %d\n",bsp->number_of_hidden_sectors);
    printf("\n");

    if (stricmp("get",argv[1])==0)
        i = do_get();
    if (stricmp("put",argv[1])==0)
        i = do_put(argc-3,argv[3],argv[4],argv[5]);
    _bios_disk(_DISK_RESET,&info);
    return i;
    }
